#!/usr/bin/env python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# +------------------------------------------------------------------+
# |             ____ _               _        __  __ _  __           |
# |            / ___| |__   ___  ___| | __   |  \/  | |/ /           |
# |           | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /            |
# |           | |___| | | |  __/ (__|   <    | |  | | . \            |
# |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
# |                                                                  |
# | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
# +------------------------------------------------------------------+
#
# This file is part of Check_MK.
# The official homepage is at http://mathias-kettner.de/check_mk.
#
# check_mk is free software;  you can redistribute it and/or modify it
# under the  terms of the  GNU General Public License  as published by
# the Free Software Foundation in version 2.  check_mk is  distributed
# in the hope that it will be useful, but WITHOUT ANY WARRANTY;  with-
# out even the implied warranty of  MERCHANTABILITY  or  FITNESS FOR A
# PARTICULAR PURPOSE. See the  GNU General Public License for more de-
# tails. You should have  received  a copy of the  GNU  General Public
# License along with GNU Make; see the file  COPYING.  If  not,  write
# to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
# Boston, MA 02110-1301 USA.

import os

import cmk.paths

import cmk_base.piggyback as piggyback

from .abstract import CheckMKAgentDataSource

class PiggyBackDataSource(CheckMKAgentDataSource):
    def __init__(self, hostname, ipaddress):
        super(PiggyBackDataSource, self).__init__(hostname, ipaddress)
        self._source_hostnames = set()


    def id(self):
        return "piggyback"


    def describe(self):
        path = os.path.join(cmk.paths.tmp_dir, "piggyback", self._hostname)
        return "Process piggyback data from %s" % path


    def _execute(self):
        entries = piggyback.get_piggyback_raw_data(self._hostname) \
                + piggyback.get_piggyback_raw_data(self._ipaddress)

        raw_data = ""
        for source_hostname, source_raw_data in entries:
            self._source_hostnames.add(source_hostname)
            raw_data += source_raw_data
        return raw_data


    def _get_raw_data(self):
        """Returns the current raw data of this data source

        Special for piggyback: No caching of raw data
        """
        self._logger.verbose("Execute data source")
        return self._execute(), False


    def _summary_result(self):
        """Returns useful information about the data source execution

        Return only summary information in case there is piggyback data"""

        if self._host_sections and self._host_sections.sections:
            output = "Processed from: %s" % ", ".join(self._source_hostnames)
        else:
            output = ""

        return 0, output, []
